==> v10.5.1.zip: brewman/brewman/brewman/routers/attendance_report.py <==
import csv
import io

from datetime import date, datetime

from fastapi import APIRouter
from fastapi.responses import StreamingResponse
from sqlalchemy import or_, select
from sqlalchemy.orm import Session

from ..db.session import SessionFuture
from ..models.attendance import Attendance
from ..models.attendance_type import AttendanceType
from ..models.employee import Employee
from .attendance import date_range


router = APIRouter()


@router.get("", response_class=StreamingResponse)
def get_report(
    s: str,
    f: str,
    # user: UserToken = Security(get_user, scopes=["attendance"]) ## removed as jwt headers are a pain in the ass
) -> StreamingResponse:
    try:
        with SessionFuture() as db:
            output = io.StringIO()
            attendance_record(
                datetime.strptime(s, "%d-%b-%Y"),
                datetime.strptime(f, "%d-%b-%Y"),
                output,
                db,
            )
            headers = {"Content-Disposition": "attachment; filename = attendance-record.csv"}
            output.seek(0)
            return StreamingResponse(output, media_type="text/csv", headers=headers)
    finally:
        pass
        # output.close()


def attendance_record(start_date: date, finish_date: date, output: io.StringIO, db: Session) -> None:
    header = ["Code", "Name", "Designation", "Department", "Salary", "Points"]
    for date_ in date_range(start_date, finish_date, inclusive=True):
        header.append(date_.strftime("%d-%b"))

    not_set = AttendanceType.by_id(0)
    employees = (
        db.execute(
            select(Employee)
            .where(Employee.joining_date <= finish_date, or_(Employee.is_active, Employee.leaving_date >= start_date))
            .order_by(Employee.cost_centre_id, Employee.designation, Employee.name)
        )
        .scalars()
        .all()
    )

    writer = csv.writer(output)
    writer.writerow(header)
    for employee in employees:
        row_display = [
            employee.code,
            employee.name,
            employee.designation,
            employee.cost_centre.name,
            employee.salary,
            employee.points,
        ]
        row_value = ["", "", "", "", employee.salary, employee.points]
        for date_ in date_range(start_date, finish_date, inclusive=True):
            att = (
                db.execute(
                    select(Attendance).where(
                        Attendance.employee_id == employee.id,
                        Attendance.date == date_,
                        Attendance.is_valid == True,  # noqa: E712
                    )
                )
                .scalars()
                .one_or_none()
            )
            att = not_set if att is None else AttendanceType.by_id(att.attendance_type)
            row_display.append(att.name)
            row_value.append(att.value)
        writer.writerow(row_display)
        writer.writerow(row_value)

==> v11.6.0.zip: brewman/brewman/brewman/routers/attendance_report.py <==
import csv
import io

from datetime import date, datetime

from fastapi import APIRouter
from fastapi.responses import StreamingResponse
from sqlalchemy import or_, select
from sqlalchemy.orm import Session

from ..db.session import SessionFuture
from ..models.attendance import Attendance
from ..models.attendance_type import AttendanceType
from ..models.employee import Employee
from .attendance import date_range


router = APIRouter()


@router.get("", response_class=StreamingResponse)
def get_report(
    s: str,
    f: str,
    # user: UserToken = Security(get_user, scopes=["attendance"]) ## removed as jwt headers are a pain in the ass
) -> StreamingResponse:
    try:
        with SessionFuture() as db:
            output = io.StringIO()
            attendance_record(
                datetime.strptime(s, "%d-%b-%Y"),
                datetime.strptime(f, "%d-%b-%Y"),
                output,
                db,
            )
            headers = {"Content-Disposition": "attachment; filename = attendance-record.csv"}
            output.seek(0)
            return StreamingResponse(output, media_type="text/csv", headers=headers)
    finally:
        pass
        # output.close()


def attendance_record(start_date: date, finish_date: date, output: io.StringIO, db: Session) -> None:
    header = ["Code", "Name", "Designation", "Department", "Salary", "Points"]
    for date_ in date_range(start_date, finish_date, inclusive=True):
        header.append(date_.strftime("%d-%b"))

    not_set = AttendanceType.by_id(0)
    employees = (
        db.execute(
            select(Employee)
            .where(Employee.joining_date <= finish_date, or_(Employee.is_active, Employee.leaving_date >= start_date))
            .order_by(Employee.cost_centre_id, Employee.designation, Employee.name)
        )
        .scalars()
        .all()
    )

    writer = csv.writer(output)
    writer.writerow(header)
    for employee in employees:
        row_display = [
            employee.code,
            employee.name,
            employee.designation,
            employee.cost_centre.name,
            employee.salary,
            employee.points,
        ]
        row_value = ["", "", "", "", employee.salary, employee.points]
        for date_ in date_range(start_date, finish_date, inclusive=True):
            att = (
                db.execute(
                    select(Attendance).where(
                        Attendance.employee_id == employee.id,
                        Attendance.date_ == date_,
                        Attendance.is_valid == True,  # noqa: E712
                    )
                )
                .scalars()
                .one_or_none()
            )
            at_type = not_set if att is None else AttendanceType.by_id(att.attendance_type)
            row_display.append(at_type.name)
            row_value.append(at_type.value)
        writer.writerow(row_display)
        writer.writerow(row_value)
